!*********************************************
module modname
! module with some definitions to process command line and parameter
! file. The remaining is in module parametri.
use nrtype

character(80)::name,name_dir
! chiavi che separano le sezioni di nome.par
character(len=10), dimension(3)::seckeys=(/"MAIN   ","IMF    ","ANALY  "/)

character(80), dimension(:), allocatable:: opt
integer,save:: nopt
! opt is an array (to be allocated to nopt elements) of strings for command 
! line options


contains

    LOGICAL FUNCTION search_key_in_file(unit,key)
    ! legge righe nel'unita' unit finche' trova key.
    ! se non trova restituisce false, altrimenti true      
    implicit none
    integer(i4b)::unit
    character(*):: key
    character(80)::riga

    riga="&%#"
    search_key_in_file = .false.

    do while (riga/=key)
        read(unit,*,err=999,end=999) riga 
        if (riga==key) then
	        search_key_in_file = .true.
        endif
    enddo
999 continue
    end function search_key_in_file

!**********
    subroutine skip_lines_in_file(unit,rem)
    ! salta righe nel'unita' unit fintantoche' cominciano con uno
    ! dei caratteri della stringa rem.
    implicit none
    integer(i4b)::unit
    character(*):: rem
    character(1)::first

    first=rem(1:1)

    do while (scan(first,rem)/=0) 
        read(unit,*,err=999,end=999) first 
    enddo
    backspace(unit)

999 continue
    end subroutine skip_lines_in_file


!**********
    function arg_empty(arg)
    ! tests if the command line argument in the string arg is
    ! empty. This complication arises because in this case 
    ! win95/nt getarg returns a string of char(0) (nul) whilst 
    ! on unix a string of spaces.
    implicit none
    character (len=*),intent(in)::arg
    logical::arg_empty

    arg_empty=(arg(1:1)==char(0).or.arg(1:1)==' ')

    end function arg_empty


end module modname


!*********************************************

module lifetime
! module for lifetime of stars as a function of m and Z,
! mass of stars as a function of lifetime and Z
! dm/dt as a function of m and Z
use nrtype
use nrutil, only:nrerror
use nr, only:hunt
use nr90_gian, only:biliin, liin
use modname

real(sp), allocatable, save, dimension(:) :: zarr,marr
real(sp), allocatable, save, dimension(:,:) :: tarr,dmdtarr,tdmarr
integer(I4B):: nz=0, nm=0

real(sp), save:: to_yea=1.0e9


!**********
contains

    function life(m_or_t,z,flag)
    implicit none
    ! returns:
    ! lifetime of star of mass m, metallicity z if flag=0 or not present
    ! mass of star of lifetime t, metallicity z if flag=1
    ! dm/dt of t, z if flag=2
    ! masses in solar units.

    ! on first call, reads a data file "lifetime.dat"
    ! with lifetime as a function of m (rows) and z (columns)
    ! use allocatable arrays, thus the data file can be
    ! replaced without recompilation. Its format is:
    ! first row for comments, second for nz and nm, 
    ! third row zarr, then lifetimes. Sample:
!# second row #M, #Z,  third row Z, then M,tau(m,Z_i)
!5  30
!        0.0004      0.004       0.008       0.02         0.05
! 0.6    4.28E+10    5.35E+10    6.47E+10    7.92E+10    7.18E+10  
! 0.7    2.37E+10    2.95E+10    3.54E+10    4.45E+10    4.00E+10  
! and so on..
    ! NOTE: if argument z is outside tabulated limits???sto studiando, then
    ! the value returned correspond to that at the nearest bound.
    ! By converse, on t or m the values are extrapolated.

    real(sp)::life
    real(sp),intent(in)::m_or_t,z
    integer(i4b),optional,intent(in)::flag

    integer(i4b), save::i
    integer(I4B):: alloerr, caso
    real(sp):: y,y1,y2,zwk,mwk,twk
    
    if (present(flag)) then
        caso=flag
    else
        caso=0
    endif


    if (nz==0) then ! first call, read data file
        open(34,file=trim(name_dir)//'lifetime.dat')
        read(34,*) ; read(34,*) nz,nm
        allocate(zarr(nz),marr(nm),tarr(nm,nz), &
        dmdtarr(nm-1,nz),tdmarr(nm-1,nz),stat=alloerr)
        if (alloerr/=0) then
            call nrerror('allocation error in function life')
       endif
        read(34,*) zarr
        do i=1,nm
            read(34,*) marr(i),tarr(i,:)
        enddo
        close(34)

        tarr=tarr/to_yea

! eleganti operazioni di array per calcolo rapporti incrementali
        dmdtarr=spread(marr(2:nm)-marr(1:nm-1),2,nz)/ &
                         (tarr(2:nm,:)-tarr(1:nm-1,:))
        tdmarr=(tarr(2:nm,:)+tarr(1:nm-1,:))/2



    endif

!	zwk=z
    zwk=min(zwk,maxval(zarr))
    zwk=max(zwk,minval(zarr))

    select case (caso)

    case (0)
    ! lifetime(m,z)
        !mwk=m_or_t
		mwk=min(m_or_t,maxval(marr))
        mwk=max(mwk,minval(marr))
        call biliin(marr,zarr,tarr,mwk,zwk,y)
    case (1)
    ! mass(t,z)
        call hunt(zarr,zwk,i)
        i=min(i,nz-1)
		i=max(i,1)

        twk=min(m_or_t,maxval(tarr(:,i:i+1)))
        twk=max(twk,minval(tarr(:,i:i+1)))

!        twk=m_or_t
!        if ( (twk>max(tarr(1,i),tarr(1,i+1))) .or. &
!        (twk<min(tarr(nm,i),tarr(nm,i+1))) ) then
!            y=0
!        else
            call liin(tarr(:,i),marr,twk,y1)        
            call liin(tarr(:,i+1),marr,twk,y2)        
            y=y1+(y2-y1)*(zwk-zarr(i))/(zarr(i+1)-zarr(i))
!        endif

    case (2)
    ! dm/dt(t,z) 

        call hunt(zarr,zwk,i)
		i=max(i,1)
        i=min(i,nz-1)

        twk=min(m_or_t,maxval(tdmarr(:,i:i+1)))
        twk=max(twk,minval(tdmarr(:,i:i+1)))

!        twk=m_or_t
!        if ( (twk>max(tdmarr(1,i),tdmarr(1,i+1))) .or. &
!        (twk<min(tdmarr(nm-1,i),tdmarr(nm-1,i+1))) ) then
!            y=0
!        else
            call liin(tdmarr(:,i),dmdtarr(:,i),twk,y1)        
            call liin(tdmarr(:,i+1),dmdtarr(:,i+1),twk,y2)        
            y=y1+(y2-y1)*(zwk-zarr(i))/(zarr(i+1)-zarr(i))
!        endif


    case default
        call nrerror('invalid flag in function life')
    end select


    life=y


    end function life

!**********
    function erre(m,z,i) ! Laura simply says "e..e"
    implicit none
!   returns mass fraction of star with mass m, metallicity z
!   given back to ISM at the end of life in the form of element
!   i
    real(sp)::erre
    real(sp),intent(in)::m,z
    integer(i4b),intent(in)::i
    

    integer, parameter::nm=25,ne=3  !nm=37
    real(sp),save:: masarr(nm), errarr(nm,ne)
    logical,save::firstcall=.true.
    integer:: irea,jrea
    real::res
   
    if (firstcall) then
        open(77,file=trim(name_dir)//'r.dat') 
        read(77,*); read(77,*)
        do irea=1,nm
            read(77,*) masarr(irea),(errarr(irea,jrea),jrea=1,ne)
        enddo
        close(77)
        firstcall=.false.
    endif
   
    call liin(masarr,errarr(:,i),m,res)        
       
    erre=res


    end function erre



end module lifetime




!*********************************************
module mod_imf
! module for Initial Mass Function, in the form piecewise
! power laws of mass:
!   imf(m) = aarr(i) m**xarr(i) if mbarr(i-1) < m < mbarr(i)
 
use nrtype
use nrutil, only:nrerror
use nr, only:hunt

use lifetime, only: life, zarr, nz
use modname

integer(i4b),save ::n_imf=0 ! number of pieces
real(sp), allocatable, save, dimension(:) ::a_arr,mb_arr,x_arr
! normalization, upper limit and index for each interval
real(sp), allocatable, save, dimension(:), PRIVATE ::fra_arr,franum_arr
! mass fraction and number fraction of stars in each piece;
! useful to compute integrals of imf and imf/m (number)
! between two given masses
real(sp), save::m_l,m_u ! minimum and maximum mass: 
! m_l=mbarr(0), m_u=mbarr(n_imf)
real(sp), save::t_lif_min,t_lif_max ! minimum and maximum mass: 
! lifetimes of stars corresponding to m_u and m_l respectively,
! and considering all metallicities (in life)


!**********
contains

    function imf(m,filename,nor)
!   imf(m) = aarr(i) m**xarr(i) if mbarr(i-1) < m < mbarr(i)
!   if filename is present, then n_imf and arrays xarr and mbarr
!   are read from a file and aarr are computed to ensure 
!   continuity at mbarr(i) and normalization to 1. Thus the
!   first call must be done with the name of the desired
!   file as second argument. 
!   if nor is present it is set to the integral of the IMF
!   BEFORE normalization. Useful for spectrophotometric code.
!  
!   Example of file follows:
!Parameters for test IMF. Rows content: 1-n_imf 2-mbarr 3-xarr 
!3
!0.1 3 30 100
!1.35 1.5 2.0

    implicit none
    real(sp)::imf    
    real(sp),intent(in)::m    
    character(*), optional, intent(in):: filename
    real(sp),optional,intent(out) ::nor
    
    integer(i4b)::alloerr,i
    integer(i4b),save::j
    real(sp) ::norm
    real(sp),allocatable,dimension(:)::wk
    character(80)::comment_line

    if (present(filename)) then
            open(34,file=filename)

            if (.not.search_key_in_file(34,seckeys(2))) then
                 write(*,*) "Key IMF not found in file ",filename
	             stop
            endif

            read(34,'(a80)') comment_line
            write(*,'(1x,a80)') comment_line
            read(34,*) n_imf
            write(*,*) 'n_imf: ',n_imf

            allocate(a_arr(n_imf),mb_arr(0:n_imf),x_arr(n_imf), &
            fra_arr(n_imf),franum_arr(n_imf),wk(n_imf),stat=alloerr)
            if (alloerr/=0) then
                call nrerror('allocation error in function IMF')
            endif

            a_arr=0; mb_arr=0 ; x_arr=0; fra_arr=0; 
            franum_arr=0; wk=0

            read(34,*) mb_arr
            write(*,*) 'mb_arr: ',mb_arr
            read(34,*) x_arr
            write(*,*) 'x_arr:  ',x_arr
            close(34)
            m_l=mb_arr(0)
            m_u=mb_arr(n_imf)
            
            ! continuity condition at mbarr
            a_arr(1)=1
            do i=1,n_imf-1
                a_arr(i+1)=a_arr(i)*mb_arr(i)**(x_arr(i+1)-x_arr(i))
            enddo
            wk=1-x_arr

            ! normalization to 
            !   /
            !  | IMF(m) dm =1
            ! /
            where (x_arr/=1)
                fra_arr=a_arr*(mb_arr(1:n_imf)**wk-mb_arr(0:n_imf-1)**wk)/wk
            elsewhere
                fra_arr=a_arr*log(mb_arr(1:n_imf)/mb_arr(0:n_imf-1))
            endwhere
            norm=sum(fra_arr)
            fra_arr=fra_arr/norm

            ! useful to compute integrals phi/m (number fraction)
            franum_arr=a_arr* &
            (mb_arr(0:n_imf-1)**(-x_arr)-mb_arr(1:n_imf)**(-x_arr)) &
            / x_arr
            franum_arr=franum_arr/norm


            a_arr=a_arr/norm

            if (present(nor)) then
                nor=norm
            endif

            ! computation of t_lif_min and t_lif_max
            norm=life(1.0,0.02)  ! initializing call to life,
            ! in order to get zarr allocated

            ! ammetto che scrivere la seguente istruzione 
            ! mi ha arrecato un piacere paragonabile solo ad
            ! una bella scopata!!! 
            t_lif_min=minval( (/(life(m_u,zarr(i)),i=1,nz)/) )
            t_lif_max=maxval( (/(life(m_l,zarr(i)),i=1,nz)/) )


    else
        if (n_imf==0) then
            call nrerror('function IMF must be initialized')
        endif
    endif

!   actual computation 
    if ((m<m_l).or.(m>m_u)) then
         imf=0
    else
        if (n_imf==1) then
            imf=a_arr(1)*m**(-x_arr(1))
        else
            call hunt(mb_arr,m,j)
            imf=a_arr(j)*m**(-x_arr(j))
        endif
    endif

    end function imf




!***********
    function fra_imf_below(m)
    real fra_imf_below
    integer(i4b)::j
    real(sp)::m

    real(sp)::wk

    if (n_imf==0) then
        call nrerror('function IMF must be initialized before fra_imf_below')
    endif


    if (m<m_l) then
        fra_imf_below=0
    else  if (m>m_u) then
        fra_imf_below=1
    else
        call hunt(mb_arr,m,j)
        wk=1-x_arr(j)
        tot=a_arr(j)*(m**wk-mb_arr(j-1)**wk)/wk

        if (j>1) then
            tot=tot+sum(fra_arr(1:j-1))
        endif
        fra_imf_below=tot
    endif
    

    end function fra_imf_below


end module mod_imf

!*********************************************
module schmidt
use nrtype

real(sp), save:: nu_sch, k_sch

contains

!***********
    function sfr_sch(Mg)
        implicit none
        real(sp), intent(in)::Mg
        real(sp):: sfr_sch

        sfr_sch = nu_sch * Mg ** k_sch

    end function sfr_sch

end module schmidt


!*********************************************
module infall
use nrtype

real(sp), save:: A_inf, tau_inf

contains

!***********
    function inf(t)
        implicit none
        real(sp), intent(in)::t
        real(sp):: inf

        inf = A_inf * exp (-t/tau_inf)

    end function inf

end module infall


!*********************************************
module checod_glob
! module daclaring main data structures for chemical code
use nrtype
use nrutil, only:nrerror
use nr90_gian, only:liin
!use nr, only:hunt

integer(i4b),save ::i_sfr_m ! index regulating dependence of SF on mass
! may be >=<0, the behaviour before t_win depends only on abs(i_sfr_m) 
! after t_win SF is stopped for positive values while for negative 
! values continues with same dependence on masses 
 integer(i4b),save ::i_sfr_t ! index regulating dependence of SF on time

real(sp),save:: t1_sfr_t,t2_sfr_t,fra_sfr_t,tau_sfr_t,A_sfr_t=0 ! parameters for sf dependent on t ???

real(sp),save ::t_win ! time at which gas is lost and infall is stopped (wind)
! after t_win SF is stopped if i_sfr_m>0, otherwise continues with same
! dependence on masses 
real(sp),save ::f_win ! feed back is multiplied by a factor f_win 
! (hopefully between 0 and 1) after wind 
real(sp),save ::m_inf,t_inf ! mass fallen at t_inf


integer(i4b),save ::n_tim ! number of timesteps 
integer(i4b),save ::n_ele ! number of elements
integer(i4b),save ::i_now ! index of present timestep

integer(i4b),save ::i_fee_bac ! element index for wich stars feed back is
! computed
real(sp),save ::t_fee_bac ! time for wich stars feed back is
! computed
logical,save::int_flag=.true. ! controls whether feed back
! rate is always computed (false) or interpolated at times
! strictly between t and t+dt (true)
real(sp),save ::t_fin ! final time 


real(sp), allocatable, save, dimension(:) ::t_arr,sfr_arr,zet_arr,ms_arr,snr_arr
! timesteps,  sfr   and metallicity at timesteps, alive stars, SN rate
real(sp), allocatable, save, dimension(:,:) ::mg_arr,fb_arr
! gas mass and "feed back rate"
! at timesteps (first index) in element identified
! by second index. Second index 0 is for total gas mass
real(sp), allocatable, save, dimension(:) ::mg_now_arr
! current gas mass at timesteps in elements (index runs from 1) 

! quantities for SN rate computation: 
real(sp)::tmin,tmax
real(sp),parameter::snI_su_snII=0.25
real(sp)::m_up


!**********

contains 

    subroutine glob_init
    ! THIS IS VERY PRELIMINARY
    implicit none
    integer(i4b)::alloerr

    n_ele=3
    allocate(t_arr(n_tim),sfr_arr(n_tim),zet_arr(n_tim),&
    mg_now_arr(n_ele),mg_arr(n_tim,0:n_ele),fb_arr(n_tim,0:n_ele), &
    ms_arr(n_tim),snr_arr(n_tim),stat=alloerr)
    if (alloerr/=0) then
         call nrerror('allocation error in subroutine glob_init')
    endif
    t_arr=0; sfr_arr=0; zet_arr=0; mg_now_arr=0; mg_arr=0;
    fb_arr=0; ms_arr=0


    end subroutine glob_init



    !******
    subroutine glob_finish
    use lifetime, only:zarr,marr,tarr,dmdtarr,tdmarr
    implicit none

    deallocate(t_arr,sfr_arr,zet_arr,mg_now_arr,mg_arr,fb_arr)
    deallocate(zarr,marr,tarr,dmdtarr,tdmarr,ms_arr,snr_arr)


    end subroutine glob_finish


    !******
    function sfr_t(t)
        implicit none
        real(sp), intent(in)::t
        real(sp):: sfr_t,wk


		if ((t>=t1_sfr_t).and.(t<=t2_sfr_t)) then

			if (a_sfr_t==0) then ! on first call initialize normalization
				select case (i_sfr_t)
				case (1,-1)  !!!!qui
					a_sfr_t=fra_sfr_t*m_inf/(t2_sfr_t-t1_sfr_t)
				case (2,-2)  !!!!qui
					a_sfr_t=fra_sfr_t*m_inf/tau_sfr_t/ &
					(1-exp(-(t2_sfr_t-t1_sfr_t)/tau_sfr_t))
				case default
					call nrerror('invalid i_sfr_t')
				end select
			endif


			select case (i_sfr_t)
			case (1,-1)  !!!!qui
				sfr_t=a_sfr_t
			case (2,-2)  !!!!qui
				wk=a_sfr_t*exp(-(t-t1_sfr_t)/tau_sfr_t)
				sfr_t=wk
			case default
				call nrerror('invalid i_sfr_t')
			end select

        else
            sfr_t=0
		endif


	end function sfr_t



    !******
    function sfr_law(Mg,t)
        use schmidt
        use infall
        implicit none
        real(sp), intent(in)::Mg
        real(sp), intent(in)::t
        real(sp):: sfr_law,wk

		sfr_law=0

        if ((t>t_win).and.(i_sfr_m>=0).and.(i_sfr_t>=0)) then  !!!!qui
        !si stoppa tutto
            sfr_law=0
        else if ( (t<=t_win).or.( (t>t_win).and.(i_sfr_m<0).and.(i_sfr_t<0) ) ) then  !!!!qui
        !si fanno entrambe le sf prima del vento o dopo
            select case (i_sfr_m)
			case (0)
            case (1,-1)
    !       simple Schmidt
                sfr_law = sfr_law+sfr_sch(Mg)
            case (2,-2)
    !       Talbott & Arnet's modified Schmidt
                wk = sfr_sch(Mg)
                if ((k_sch/=1).and.(t>t_arr(1))) then
                    wk = wk *  &
                    ( A_inf * tau_inf * (1-exp(-t/tau_inf)) ) ** (1-k_sch)
                endif 
				sfr_law=sfr_law+wk                               
            case default
                call nrerror('invalid i_sfr_m')
            end select

			if (i_sfr_t/=0)	then
				sfr_law=sfr_law+sfr_t(t)				
			endif
        else if ( (t>t_win).and.( (i_sfr_m<0).or.(i_sfr_t<0) ) ) then  !!!!qui
        !si continua una sf dopo il vento
            select case (i_sfr_m)
            case (0,1,2)
            case (-1)
    !       simple Schmidt
               sfr_law = sfr_law+sfr_sch(Mg)
            case (-2)
    !       Talbott & Arnet's modified Schmidt
               wk = sfr_sch(Mg)
               if ((k_sch/=1).and.(t>t_arr(1))) then
                  wk = wk *  &
                  ( A_inf * tau_inf * (1-exp(-t/tau_inf)) ) ** (1-k_sch)
               endif 
			      sfr_law=sfr_law+wk                               
            case default
               call nrerror('invalid i_sfr_m')
            end select
			if (i_sfr_t<0)	then
				sfr_law=sfr_law+sfr_t(t)				
			endif            
                            
        endif

    end function sfr_law

    !******
    subroutine update_mg_zet_sfr(i_upd)
    ! update mg_arr(i_upd,0), i.e. total gas mass, zet_arr(i_upd)
    ! and sfr_arr(i_upd) (from assumed law)
    ! from mg_arr(i_upd,1:n_ele), i.e. gas mass in single elements.
    

    implicit none
    integer(i4b),intent(in)::i_upd

    mg_arr(i_upd,0)=sum(mg_arr(i_upd,1:n_ele))
    sfr_arr(i_upd)=sfr_law(mg_arr(i_upd,0),t_arr(i_upd))

    if (mg_arr(i_upd,0)/=0) then
        zet_arr(i_upd)=sum(mg_arr(i_upd,3:n_ele))/mg_arr(i_upd,0)
    else
        zet_arr(i_upd)=0
    endif

    end subroutine update_mg_zet_sfr


    !******
    function sfr(t)
!   SFR interpolated from sfr_arr at time t
    implicit none
    real(sp):: sfr
    real(sp),intent(in)::t
    integer(i4b),save::io
    if (t>t_arr(i_now)) then
         call nrerror('invalid time argument in function sfr')
    else
         call liin(t_arr(1:i_now),sfr_arr(1:i_now),t,sfr,io)
    endif

    end function sfr


    !******
    function zet(t)
!   metallicity interpolated from zet_arr at time t
    implicit none
    real(sp):: zet
    real(sp),intent(in)::t
    integer(i4b),save::io
    if (t>t_arr(i_now)) then
         call nrerror('invalid time argument in function zet')
    else
         call liin(t_arr(1:i_now),zet_arr(1:i_now),t,zet,io)
    endif

    end function zet


    !******
    function mg(t,i_ele)
    implicit none
    real(sp):: mg
    real(sp),intent(in)::t
    integer(i4b),intent(in)::i_ele

    integer(i4b),save::io
    if (t>t_arr(i_now)) then
         call nrerror('invalid time argument in function mg')
    else
         call liin(t_arr(1:i_now),mg_arr(1:i_now,i_ele),t,mg,io)
    endif

    end function mg

!***********


    FUNCTION fun_fee_bac(tp)
!   PROBABLY NOT USED
!   function to be integrated to get dM/dt due to star's feed back
!   at time t_arr(i_now) and for element i_fee_bac
    USE nrtype
    USE lifetime
    USE mod_imf

    IMPLICIT NONE
    REAL(SP), DIMENSION(:), INTENT(IN) :: tp
    REAL(SP), DIMENSION(size(tp)) :: fun_fee_bac

    REAL(SP), DIMENSION(size(tp)) :: res

    
    real(sp):: t, m, z, dmdt
    integer(i4b)::siz,i_siz

    t=t_fee_bac

    
    siz=size(tp)
    do i_siz=1, siz
        z=zet(tp(i_siz))
        m=life(t-tp(i_siz),z,1)
        dmdt=life(t-tp(i_siz),z,2)

        res(i_siz)=dmdt*sfr(tp(i_siz))*imf(m)*erre(m,z,i_fee_bac)
    enddo


    fun_fee_bac=res

    END FUNCTION fun_fee_bac


!***********
    FUNCTION fun_fee_bac_log(x)
!   function to be integrated to get dM/dt due to star's feed back
!   at time t_arr(i_now) and for element i_fee_bac
!   The integration is done in ln(look back time) since most
!   of the integral is contributed from recent times and thus
!   direct numeric integration in galactic time converges 
!   much more slowly
    USE nrtype
    USE lifetime
    USE mod_imf

    IMPLICIT NONE
    REAL(SP), DIMENSION(:), INTENT(IN) :: x
    REAL(SP), DIMENSION(size(x)) :: fun_fee_bac_log

    REAL(SP), DIMENSION(size(x)) :: res,tau,tp

    
    real(sp):: m, z, dmdt
    integer(i4b)::siz,i_siz


    tau=exp(x)
    tp=max(t_fee_bac-tau(:),t_arr(1))
    
    siz=size(tp)
    do i_siz=1, siz
        z=zet(tp(i_siz))
        m=life(tau(i_siz),z,1)
        dmdt=life(tau(i_siz),z,2)
        res(i_siz)= tau(i_siz) &
        *dmdt*sfr(tp(i_siz))*imf(m)*erre(m,z,i_fee_bac)
    enddo


    fun_fee_bac_log=res

    END FUNCTION fun_fee_bac_log



!***********
    SUBROUTINE derivs(t,mg,dmgdt)
	USE nrtype
    use nr90_gian, only:qromb_eps

    use infall
    use mod_imf, only:t_lif_max,t_lif_min
    use lifetime, only:life
	IMPLICIT NONE
	REAL(SP), INTENT(IN) :: t
	REAL(SP), DIMENSION(:), INTENT(IN) :: mg
	REAL(SP), DIMENSION(:), INTENT(OUT) :: dmgdt


	REAL(SP), DIMENSION(size(dmgdt)) :: dmgdt_inf,dmgdt_sf,dmgdt_fb
    real(sp)::in,mgtot,sf
    real(sp)::tmin,tmax,tup,f


    ! infall
    if (t>t_win) then
        in=0
    else
        in=inf(t)        
    endif

    dmgdt_inf(1)=0.8*in
    dmgdt_inf(2)=0.2*in
    dmgdt_inf(3)=1e-7*in

    ! star formation
    mgtot=sum(mg)
    if (mgtot>0) then  !!!!qui /=
        sf=sfr_law(mgtot,t)
        dmgdt_sf = - sf * mg / mgtot
    else
        dmgdt_sf=0
    endif

    if (t>t_win) then
        f=f_win
    else
        f=1.0
    endif

    fnotzero:if (f/=0) then
    ! if all gas is ejected from the system, fb computatuion is a waste of time

        ! feed back
        int: if (int_flag) then 
        ! if int_flag is true, then at each new timestep the
        ! feed back rate is firstly computed at the end of the
        ! timestep, and then linearly interpolated between the
        ! values at the begin and end of timestep 
            if (t==t_arr(i_now)) then ! timestep has changed:
            ! compute fbr at the end

                if (i_now==1) then 
                    dmgdt_fb=0
                else
                ! first dmgdt_fb is computed at t_arr(i_now+1)
                ! and placed in fb_arr(i_now+1,:), then 
                ! dmgdt_fb is set to fb_arr(i_now,:) previusly computed
                    t_fee_bac=t_arr(i_now+1)

    !   since integrand is quite irregular for young stars, probably due to dm/dt term,
    !   it turns out to be much more efficient, in order to achieve an acceptable accuracy.
    !   to split the integral.
                    tmax=t_fee_bac - t_lif_min
                    tmax=max(tmax,t_arr(1))
                    tmin=t_fee_bac-t_lif_max
                    tmin=max(tmin,t_arr(1))
                    ! since lifetime is not a strong function of z, we can evaluate at present z
                    tup=t_fee_bac-life(m_up,zet_arr(i_now),0) 
                    tup=max(tup,t_arr(1))

                    do i_fee_bac=1,n_ele
                        dmgdt_fb(i_fee_bac) = -qromb_eps(fun_fee_bac,tmin,tup,1e-4) &  
                                              -qromb_eps(fun_fee_bac,tup,tmax,1e-4)
                    enddo


                endif

                fb_arr(i_now+1,1:n_ele)=dmgdt_fb(1:n_ele)

                fb_arr(i_now+1,0)=sum(dmgdt_fb(1:n_ele))

                dmgdt_fb(1:n_ele)=fb_arr(i_now,1:n_ele)

            else
                ! linear interpolation
                dmgdt_fb(1:n_ele)=fb_arr(i_now,1:n_ele)+ &
                (fb_arr(i_now+1,1:n_ele)-fb_arr(i_now,1:n_ele))* &
                (t-t_arr(i_now))/(t_arr(i_now+1)-t_arr(i_now))
            endif
        else
        ! int_fla is false: fbr always exactly computed
        
            if (t==t_arr(i_now)) then
                dmgdt_fb(1:n_ele)=fb_arr(i_now,1:n_ele)
            else
                if (i_now==1) then
                    dmgdt_fb=0
                else
                    t_fee_bac=t
                    tmax=t_fee_bac - t_lif_min
                    tmax=max(tmax,0.0)
                    tmin=t_fee_bac-t_lif_max
                    tmin=max(tmin,0.0)

                    do i_fee_bac=1,n_ele

           dmgdt_fb(i_fee_bac) = -qromb_eps(fun_fee_bac,tmin,tmax,1e-5)
                       !dmgdt_fb(i_fee_bac) = &
                       !qromb_eps(fun_fee_bac_log,log(t-tmin),log(t-tmax),1e-4)

                    enddo
                endif
            endif
            if (t==t_arr(i_now+1)) then
                fb_arr(i_now+1,1:n_ele)=dmgdt_fb(1:n_ele)
                fb_arr(i_now+1,0)=sum(dmgdt_fb(1:n_ele))
            endif
        endif int
    endif fnotzero

    ! summing up
    
    dmgdt=dmgdt_inf+dmgdt_sf+f*dmgdt_fb

	END SUBROUTINE derivs

!***********
    FUNCTION fun_star_alive(tp)
!   function to be integrated to get the mass in stars still alive
!   at the given galactic age 
    USE nrtype
    USE lifetime
    USE mod_imf
    IMPLICIT NONE
    REAL(SP), DIMENSION(:), INTENT(IN) :: tp
    REAL(SP), DIMENSION(size(tp)) :: fun_star_alive,res

    real(sp):: t, m, z
    integer(i4b)::siz,i_siz

    siz=size(tp)

    t=t_fee_bac


    do i_siz=1, siz

        z=zet(tp(i_siz))
        m=life(t-tp(i_siz),z,1)
        res(i_siz)=sfr(tp(i_siz))*fra_imf_below(m)

    enddo

    fun_star_alive=res

    END FUNCTION fun_star_alive

!***********


    FUNCTION fun_snr(tp)
!   function to be integrated to get SN rate at time t_arr(i_now) 
    USE nrtype
    USE lifetime
    USE mod_imf

    IMPLICIT NONE
    REAL(SP), DIMENSION(:), INTENT(IN) :: tp
    REAL(SP), DIMENSION(size(tp)) :: fun_snr

    REAL(SP), DIMENSION(size(tp)) :: res

    
    real(sp):: t, m, z, dmdt
    integer(i4b)::siz,i_siz

    t=t_fee_bac

    
    siz=size(tp)
    do i_siz=1, siz
        z=zet(tp(i_siz))
        m=life(t-tp(i_siz),z,1)
        dmdt=life(t-tp(i_siz),z,2)

        if (m>0) then
            res(i_siz)=dmdt*sfr(tp(i_siz))*imf(m)/m
        else
            res(i_siz)=0
        endif
    enddo


    fun_snr=res

    END FUNCTION fun_snr



!***********






end module checod_glob



